#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "irc.h"

/*
 * This is cistrcmp except that {}|^ are lower-case for []\~.
 */
static int
irctolower(int c)
{
	if('A' <= c && c <= 'Z')
		return c+'a'-'A';
	if(c == '{')
		return '[';
	if(c == '}')
		return ']';
	if(c == '|')
		return '\\';
	if(c == '^')
		return '~';
	return c;
}

int
irccmp(char *s, char *t)
{
	int a, b;

	for(; *s || *t; s++, t++){
		a = irctolower(*s);
		b = irctolower(*t);
		if(a < b)
			return -1;
		if(a > b)
			return 1;
	}
	return 0;
}

Chan *
attach(Sess *s, char *chan)
{
	Chan *c;
	assert(s != nil);
	c = ecalloc(1, sizeof(Chan));
	c->name = estrdup(chan);
	c->next = s->chans;
	s->chans = c;
	if (ischan(c->name)){
		putln(s, "join %s", chan);
		putln(s, "who %s", chan);
	}
	return c;
}

void
detach(Sess *s, Chan *c)
{
	assert(s != nil);
	assert(c != nil);
	if (ischan(c->name))
		putln(s, "part %s\r\n", c->name);
}

int
ischan(char *s)
{
	assert(s != nil);
	switch(*s){
	case '!': return 1;
	case '#': return 1;
	case '&': return 1;
	case '+': return 1;
	}
	return 0;
}

