
enum {
	Idlesec = 120,		// idle time before an idle message may be printed
	Wraplen = 70,		// Wrap lines longer than this
};

enum {				// command types
	Cerror = -1,
	Cjoin = -2,
	Cping = -3,
	Cquit = -4,
	Cpart = -5,
	Cprivmsg = -6,
	Cnotice = -7,
	Cname = -8,		// user list
	Clist = -9,		// channel list
	Cmode = -10,
	Cnick = -11,
	Ctopic = -12,

};

typedef struct Sess Sess;
typedef struct User User;
typedef struct Chan Chan;
typedef struct Nick Nick;

struct User {
	User *next;
	char *nick;		// nickname
	char *user;		// username on home machine
	char *really;		// real name of user
	char *where;		// where their server is located
	char *host;		// users home machin/IP address
	long lastrx;		// when last message seen
	char *away;		// away message if any
};

struct Nick {
	Nick *next;
	char *nick;		// nickname
};

struct Chan {
	Chan *next;
	char *name;		// channel name
	Nick *nicks;		// nicks known on this channel
};

struct Sess {
	Chan *chans;		// channels attached
	User *users;		// user details
	long lastpr;		// when I last printed anything
	long lasttx;		// when I last sent anything
	Chan *current;		// my current channel
	char *nick;		// my nickname
	char *really;		// my real name
	char partners[64];	// to:from of last person to speak
	int away;		// am currently away
	int space;		// have whitespace

	Biobuf *bi;		// from network
	Biobuf *bo;		// to network
	Lock netlock;		// lock on network
};

Biobuf *bout;
extern int Debug;

/* chan.c */
extern int irccmp(char *, char *);
extern Chan *findchan(Sess *, char *);
extern Chan *attach(Sess *, char *);
extern void detach(Sess *, Chan *);
extern int ischan(char *);

/* io.c */
extern char *getln(Sess *);
extern int putln(Sess *, char *, ...);

/* irc.c */
extern void activity(Sess *s);
extern void idlemark(Sess *);
extern void message(Sess *, char *, char *, char *, int);
extern void netread(Sess *);
extern void consread(Sess *);

/* nicks.c */
extern void nick_add(Chan *, char *);
extern void nick_del(Chan *, char *);
extern char *nick_list(Chan *);

/* ptpc.c */
extern int ctcp_req(Sess *, char *, char *, char *, ...);
extern int ctcp_rep(Sess *, char *, char *, char *, ...);
extern void ctcp_hand(Sess *, char *, char *, char *);

/* users.c */
extern void user_add(Sess *, char *, char *, char *);
extern void user_really(Sess *, char *, char *);
extern void user_newnick(Sess *, char *, char *);
extern void user_isauth(Sess *, char *);
extern void user_away(Sess *, char *, char *);
extern void user_where(Sess *, char *, char *);
extern void user_last(Sess *, char *, long);
extern void user_del(Sess *, char *);
extern char *user_list(Sess *);
extern int user_unknown(Sess *, char *);

/* util.c */
extern char *estrdup(char *);
extern void *ecalloc(int, int);
extern char *strpfx(char *, char *);
extern char *realname(char *);
