#include <u.h>
#include <libc.h>
#include <bio.h>
#include <libsec.h>
#include <auth.h>
#include "irc.h"

int Debug = 0;
static char *Vers = "v0.1";
Biobuf boutput, *bout = &boutput;

static void
usage(void)
{
	fprint(2, "usage: %s [-id] [-k keyparam] server channel...\n", argv0);
	exits("usage");
}

void
main(int argc, char *argv[])
{
	Sess *s;
	char *keyp;
	UserPasswd *up;
	int flags, i, pid, fd;

	flags = 0;
	keyp = "";
	ARGBEGIN{
	case 'i':
		flags |= 8;
		break;
	case 'd':
		Debug++;
		break;
	case 'k':
		keyp = EARGF(usage());
		break;
	default:
		usage();
		break;
	}ARGEND;

	if (argc < 2)
		usage();

	if ((fd = dial(netmkaddr(argv[0], "tcp", "6667"), 0, 0, 0)) == -1)
		sysfatal("%s cannot dial", argv[0]);

	if ((up = auth_getuserpasswd(auth_getkey, "proto=pass service=irc %s", keyp)) == nil)
		sysfatal("factotum cannot get key %r");

	s = ecalloc(1, sizeof(Sess));
	s->bi = ecalloc(1, sizeof(Biobuf));
	s->bo = ecalloc(1, sizeof(Biobuf));
	Binit(s->bi, fd, OREAD);
	Binit(s->bo, fd, OWRITE);
	s->really = estrdup(realname(getuser()));
	s->nick = estrdup(up->user);

	Binit(bout, 1, OWRITE);

	putln(s, "user %s %d * :%s", getuser(), flags, s->really);
	putln(s, "nick %s", s->nick);
	if (*up->passwd)
		putln(s, "privmsg nickserv :identify %s", up->passwd);
	memset(up->passwd, 0, strlen(up->passwd));

	sleep(500);			// FIXME: should await authenticated message
	for (i = 1; i < argc; i++)
		s->current = attach(s, argv[i]);

	if ((pid = rfork(RFPROC|RFFDG|RFMEM)) != 0){
		consread(s);
		postnote(PNPROC, pid, "die");
	}
	else{
		netread(s);
	}

	for (i = 1; i < argc; i++)
		detach(s, s->current);
	putln(s, "quit :EOF\r\n");
	close(fd);
	exits(0);
}


