#include <u.h>
#include <libc.h>
#include <bio.h>
#include "irc.h"

/*
 * FIXME: - I haven't bothered with en-quoting or de-quoting of
 * 		\0 \r \n \001
 */
int
ctcp_rep(Sess *s, char *nick, char *tag, char *fmt, ...)
{

	int n;
	va_list arg;

	va_start(arg, fmt);
	lock(&s->netlock);
	Bprint(s->bo, "notice %s :%c%s ", nick, 1, tag);
	n = Bvprint(s->bo, fmt, arg);
	Bprint(s->bo, "%c\r\n", 1);
	Bflush(s->bo);
	if (Debug){
		fprint(2, "tx: notice %s :\\001%s ", nick, tag);
		vfprint(2, fmt, arg);
		fprint(2, "\\001\n");
	}
	unlock(&s->netlock);
	va_end(arg);
 	return n;
}

int
ctcp_req(Sess *s, char *nick, char *tag, char *fmt, ...)
{

	int n;
	va_list arg;

	va_start(arg, fmt);
	lock(&s->netlock);
	Bprint(s->bo, "notice %s :%c%s ", nick, 1, tag);
	n = Bvprint(s->bo, fmt, arg);
	Bprint(s->bo, "%c\r\n", 1);
	Bflush(s->bo);
	if (Debug){
		fprint(2, "tx: privmsg %s :\\001%s ", nick, tag);
		vfprint(2, fmt, arg);
		fprint(2, "\\001\n");
	}
	unlock(&s->netlock);
	va_end(arg);
 	return n;
}

void
ctcp_hand(Sess *s, char *src, char *dst, char *msg)
{
	long delta;
	char *p, *now;
	static long last = 0;
	static int count = 0;

	delta = time(nil)-last;
	count = (count+1)/((delta == 0)? 1: delta);

	last = time(nil);
	if (count > 5){
		ctcp_rep(s, src, "ERRMSG", "flood detected, try again");
		return;
	}

	if ((p = strpfx(msg, "ACTION ")) != nil){
		message(s, src, dst, p, 1);
		return;
	}
	if ((p = strpfx(msg, "PING ")) != nil){
		ctcp_rep(s, src, "PONG", "%Z", p);
		return;
	}
	if ((p = strpfx(msg, "ERRMSG ")) != nil){
		ctcp_rep(s, src, "ERRMSG", "%s:no error", p);
		return;
	}
	if (strpfx(msg, "VERSION") != nil){
		ctcp_rep(s, src, "VERSION", "%Z:v0.1:Plan9 Rel4", argv0);
		return;
	}
	if (strpfx(msg, "USERINFO") != nil){
		ctcp_rep(s, src, "USERINFO", "%Z, idle for %ld secs",
			s->really, time(nil) - s->lasttx);
		return;
	}
	if (strpfx(msg, "CLIENTINFO") != nil){
		ctcp_rep(s, src, "CLIENTINFO",
			"ACTION PING VERSION USERINFO CLIENTINFO TIME");
		return;
	}
	if (strpfx(msg, "TIME") != nil){
		now = ctime(time(nil));
		if ((p = strrchr(now, '\n')) != nil)
			*p = 0;
		ctcp_rep(s, src, "TIME", "%Z", now);
		return;
	}
	ctcp_rep(s, src, "ERRMSG", "unsupported request");
}
