#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ip.h>
#include <ndb.h>
#include "irc.h"

enum {
	Resolve = 0
};

void
user_add(Sess *s, char *nick, char *user, char *host)
{
	User *u;
	uchar buf[IPaddrlen];

	if(user[0] == 'i' || user[0] == 'n' && user[1] == '=')
		user += 2;

	for (u = s->users; u; u = u->next)
		if (irccmp(u->nick, nick) == 0)
			return;

	u = ecalloc(1, sizeof(User));
	u->nick = estrdup(nick);
	u->user = estrdup(user);

	if(Resolve){ 		/* This can make things very slow if DNS gets very slow */
		if(parseip(buf, host) == -1)
			u->host = estrdup(host);
		else
		if((u->host = csgetvalue("/net", "ip", host, "dom", nil)) == nil)
			u->host = estrdup(host);
	}
	else
		u->host = estrdup(host);

	u->lastrx = 0;
	u->next = s->users;
	s->users = u;		
}

void
user_really(Sess *s, char *nick, char *really)
{
	User *u;
	for (u = s->users; u; u = u->next)
		if (irccmp(u->nick, nick) == 0){
			free(u->really);
			u->really = strdup(really);
			return;
		}
}

void
user_away(Sess *s, char *nick, char *away)
{
	User *u;
	for (u = s->users; u; u = u->next)
		if (irccmp(u->nick, nick) == 0){
			free(u->away);
			u->away = nil;
			if (away)
				u->away = strdup(away);
			return;
		}
}

void
user_newnick(Sess *s, char *nick, char *newnick)
{
	User *u;
	for (u = s->users; u; u = u->next)
		if (irccmp(u->nick, nick) == 0){
			free(u->nick);
			u->nick = strdup(newnick);
			return;
		}
}

int
user_unknown(Sess *s, char *nick)
{
	User *u;
	for (u = s->users; u; u = u->next)
		if (strcmp(u->nick, nick) == 0)
			return (u->really == nil)? 1: 0;
	return 1;
}

void
user_where(Sess *s, char *nick, char *where)
{
	User *u;
	for (u = s->users; u; u = u->next)
		if (strcmp(u->nick, nick) == 0){
			free(u->where);
			u->where = strdup(where);
			return;
		}
}

void
user_last(Sess *s, char *nick, long last)
{
	User *u;
	for (u = s->users; u; u = u->next)
		if (strcmp(u->nick, nick) == 0){
			u->lastrx = last;
			return;
		}
}


void
user_del(Sess *s, char *nick)
{
	User *p, *u;
	for (p = nil, u = s->users; u; p = u, u = u->next)
		if (irccmp(u->nick, nick) == 0){
			if (p)
				p->next = u->next;
			else
				s->users = u->next;
			free(u->nick);
			free(u->user);
			free(u->host);
			free(u->really);
		}
}

static int
cmp(void *a, void *b)
{
	User **x = (User **)a;
	User **y = (User **)b;
	if((*x)->lastrx == 0)
		return 0x7fffffff;
	return  (*y)->lastrx - (*x)->lastrx;
}


char *
user_list(Sess *s)
{
	Tm *tm;
	int i, n;
	char *idle;
	User *u, **idx;
	Fmt fmt, *f = &fmt;

	n = 0;
	for (u = s->users; u; u = u->next)
		n++;
	idx = ecalloc(sizeof(User *), n);

	i = 0;
	for (u = s->users; u; u = u->next)
		idx[i++] = u;
	qsort(idx, n, sizeof(User *), cmp);

	fmtstrinit(f);
	for (i = 0; i < n; i++){
		u = idx[i];

		tm = localtime(time(nil) - u->lastrx);
		if(u->lastrx == 0)
			idle = estrdup("");
		else
		if (tm->yday)
			idle = smprint("%3d days", tm->yday);
		else
		if (u->away)
			idle = estrdup("away");
		else
			idle = smprint("%2d:%02d:%02d", tm->hour, tm->min, tm->sec);

		fmtprint(f, "%-16s %-24s %8s %s\n",
			u->nick, u->really, idle, u->where);
		free(idle);
	}
	free(idx);
	return fmtstrflush(f);
}

